@@ -3,11 +3,11 @@
     click
     ~~~~~
 
-    click is a simple Python module that wraps the stdlib's optparse to make
-    writing command line scripts fun.  Unlike other modules it's based around
+    Click is a simple Python module that wraps the stdlib's optparse to make
+    writing command line scripts fun.  Unlike other modules, it's based around
     a simple API that does not come with too much magic and is composable.
 
-    In case optparse ever goes away from the stdlib it will be shipped by
+    In case optparse ever gets removed from the stdlib, it will be shipped by
     this module.
 
     :copyright: (c) 2014 by Armin Ronacher.
@@ -29,13 +29,16 @@ from .types import ParamType, File, Path, Choice, IntRange, STRING, INT, \
 
 # Utilities
 from .utils import echo, get_binary_stream, get_text_stream, \
-     format_filename
+     format_filename, get_app_dir
 
 # Terminal functions
-from .termui import prompt, confirm, get_terminal_size, echo_via_pager
+from .termui import prompt, confirm, get_terminal_size, echo_via_pager, \
+     progressbar, clear, style, unstyle, secho, edit, launch, getchar, \
+     pause
 
 # Exceptions
-from .exceptions import ClickException, UsageError, FileError, Abort
+from .exceptions import ClickException, UsageError, BadParameter, \
+     FileError, Abort
 
 # Formatting
 from .formatting import HelpFormatter, wrap_text
@@ -60,12 +63,16 @@ __all__ = [
 
     # Utilities
     'echo', 'get_binary_stream', 'get_text_stream', 'format_filename',
+    'get_app_dir',
 
     # Terminal functions
     'prompt', 'confirm', 'get_terminal_size', 'echo_via_pager',
+    'progressbar', 'clear', 'style', 'unstyle', 'secho', 'edit', 'launch',
+    'getchar', 'pause',
 
     # Exceptions
-    'ClickException', 'UsageError', 'FileError', 'Abort',
+    'ClickException', 'UsageError', 'BadParameter', 'FileError',
+    'Abort',
 
     # Formatting
     'HelpFormatter', 'wrap_text',
@@ -73,3 +80,6 @@ __all__ = [
     # Parsing
     'OptionParser',
 ]
+
+
+__version__ = '2.0'
