@@ -1,12 +1,17 @@
 import re
 import io
+import os
 import sys
 import codecs
+from weakref import WeakKeyDictionary
 
 
 PY2 = sys.version_info[0] == 2
 
 
+_ansi_re = re.compile('\033\[((?:\d|;)*)([a-zA-Z])')
+
+
 def _make_text_stream(stream, encoding, errors):
     if encoding is None:
         encoding = get_best_encoding(stream)
@@ -38,16 +43,22 @@ class _NonClosingTextIOWrapper(io.TextIOWrapper):
         io.TextIOWrapper.__init__(self, _FixupStream(stream),
                                   encoding, errors, **extra)
 
-    # The io module is already a place where Python 2 got the
-    # python 3 text behavior forced on, so we need to unbreak
-    # it to look like python 2 stuff.
+    # The io module is a place where the Python 3 text behavior
+    # was forced upon Python 2, so we need to unbreak
+    # it to look like Python 2.
     if PY2:
         def write(self, x):
-            return io.TextIOWrapper.write(self, unicode(x))
+            if isinstance(x, str) or is_bytes(x):
+                try:
+                    self.flush()
+                except Exception:
+                    pass
+                return self.buffer.write(str(x))
+            return io.TextIOWrapper.write(self, x)
 
         def writelines(self, lines):
-            lines = map(unicode, lines)
-            return io.TextIOWrapper.writelines(self, lines)
+            for line in lines:
+                self.write(line)
 
     def __del__(self):
         try:
@@ -58,7 +69,7 @@ class _NonClosingTextIOWrapper(io.TextIOWrapper):
 
 class _FixupStream(object):
     """The new io interface needs more from streams than streams
-    traditionally implement.  As such this fixup stuff is necessary in
+    traditionally implement.  As such, this fix-up code is necessary in
     some circumstances.
     """
 
@@ -72,6 +83,11 @@ class _FixupStream(object):
         f = getattr(self._stream, 'read1', None)
         if f is not None:
             return f(size)
+        # We only dispatch to readline instead of read in Python 2 as we
+        # do not want cause problems with the different implementation
+        # of line buffering.
+        if PY2:
+            return self._stream.readline(size)
         return self._stream.read(size)
 
     def readable(self):
@@ -114,20 +130,47 @@ if PY2:
     raw_input = raw_input
     string_types = (str, unicode)
     iteritems = lambda x: x.iteritems()
+    range_type = xrange
+
+    def is_bytes(x):
+        return isinstance(x, (buffer, bytearray))
 
     _identifier_re = re.compile(r'^[a-zA-Z_][a-zA-Z0-9_]*$')
 
+    # For Windows, we need to force stdout/stdin/stderr to binary if it's
+    # fetched for that.  This obviously is not the most correct way to do
+    # it as it changes global state.  Unfortunately, there does not seem to
+    # be a clear better way to do it as just reopening the file in binary
+    # mode does not change anything.
+    #
+    # An option would be to do what Python 3 does and to open the file as
+    # binary only, patch it back to the system, and then use a wrapper
+    # stream that converts newlines.  It's not quite clear what's the
+    # correct option here.
+    if sys.platform == 'win32':
+        import msvcrt
+        def set_binary_mode(f):
+            try:
+                fileno = f.fileno()
+            except Exception:
+                pass
+            else:
+                msvcrt.setmode(fileno, os.O_BINARY)
+            return f
+    else:
+        set_binary_mode = lambda x: x
+
     def isidentifier(x):
         return _identifier_re.search(x) is not None
 
     def get_binary_stdin():
-        return sys.stdin
+        return set_binary_mode(sys.stdin)
 
     def get_binary_stdout():
-        return sys.stdout
+        return set_binary_mode(sys.stdout)
 
     def get_binary_stderr():
-        return sys.stderr
+        return set_binary_mode(sys.stderr)
 
     def get_text_stdin(encoding=None, errors=None):
         return _make_text_stream(sys.stdin, encoding, errors)
@@ -147,16 +190,20 @@ else:
     text_type = str
     raw_input = input
     string_types = (str,)
+    range_type = range
     isidentifier = lambda x: x.isidentifier()
     iteritems = lambda x: iter(x.items())
 
+    def is_bytes(x):
+        return isinstance(x, (bytes, memoryview, bytearray))
+
     def _is_binary_reader(stream, default=False):
         try:
             return isinstance(stream.read(0), bytes)
         except Exception:
             return default
             # This happens in some cases where the stream was already
-            # closed.  In this case we assume the defalt.
+            # closed.  In this case, we assume the default.
 
     def _is_binary_writer(stream, default=False):
         try:
@@ -172,16 +219,15 @@ else:
 
     def _find_binary_reader(stream):
         # We need to figure out if the given stream is already binary.
-        # This can happen because the official docs recommend detatching
+        # This can happen because the official docs recommend detaching
         # the streams to get binary streams.  Some code might do this, so
         # we need to deal with this case explicitly.
-        is_binary = _is_binary_reader(stream, False)
-
-        if is_binary:
+        if _is_binary_reader(stream, False):
             return stream
 
         buf = getattr(stream, 'buffer', None)
-        # Same situation here, this time we assume that the buffer is
+
+        # Same situation here; this time we assume that the buffer is
         # actually binary in case it's closed.
         if buf is not None and _is_binary_reader(buf, True):
             return buf
@@ -196,13 +242,13 @@ else:
 
         buf = getattr(stream, 'buffer', None)
 
-        # Same situation here, this time we assume that the buffer is
+        # Same situation here; this time we assume that the buffer is
         # actually binary in case it's closed.
         if buf is not None and _is_binary_writer(buf, True):
             return buf
 
     def _stream_is_misconfigured(stream):
-        """A stream is misconfigured if it's encoding is ASCII."""
+        """A stream is misconfigured if its encoding is ASCII."""
         return is_ascii_encoding(getattr(stream, 'encoding', None))
 
     def _is_compatible_text_stream(stream, encoding, errors):
@@ -213,7 +259,7 @@ else:
         if stream_encoding == encoding and stream_errors == errors:
             return True
 
-        # Otherwise it's only a compatible stream if we did not ask for
+        # Otherwise, it's only a compatible stream if we did not ask for
         # an encoding.
         if encoding is None:
             return stream_encoding is not None
@@ -224,24 +270,24 @@ else:
         if _is_binary_reader(text_reader, False):
             binary_reader = text_reader
         else:
-            # If there is no target encoding set we need to verify that the
-            # reader is actually not misconfigured.
+            # If there is no target encoding set, we need to verify that the
+            # reader is not actually misconfigured.
             if encoding is None and not _stream_is_misconfigured(text_reader):
                 return text_reader
 
             if _is_compatible_text_stream(text_reader, encoding, errors):
                 return text_reader
 
-            # If the reader has no encoding we try to find the underlying
+            # If the reader has no encoding, we try to find the underlying
             # binary reader for it.  If that fails because the environment is
             # misconfigured, we silently go with the same reader because this
-            # is too common to happen.  In that case mojibake is better than
+            # is too common to happen.  In that case, mojibake is better than
             # exceptions.
             binary_reader = _find_binary_reader(text_reader)
             if binary_reader is None:
                 return text_reader
 
-        # At this point we default the errors to replace instead of strict
+        # At this point, we default the errors to replace instead of strict
         # because nobody handles those errors anyways and at this point
         # we're so fundamentally fucked that nothing can repair it.
         if errors is None:
@@ -252,24 +298,24 @@ else:
         if _is_binary_writer(text_writer, False):
             binary_writer = text_writer
         else:
-            # If there is no target encoding set we need to verify that the
-            # writer is actually not misconfigured.
+            # If there is no target encoding set, we need to verify that the
+            # writer is not actually misconfigured.
             if encoding is None and not _stream_is_misconfigured(text_writer):
                 return text_writer
 
             if _is_compatible_text_stream(text_writer, encoding, errors):
                 return text_writer
 
-            # If the writer has no encoding we try to find the underlying
+            # If the writer has no encoding, we try to find the underlying
             # binary writer for it.  If that fails because the environment is
             # misconfigured, we silently go with the same writer because this
-            # is too common to happen.  In that case mojibake is better than
+            # is too common to happen.  In that case, mojibake is better than
             # exceptions.
             binary_writer = _find_binary_writer(text_writer)
             if binary_writer is None:
                 return text_writer
 
-        # At this point we default the errors to replace instead of strict
+        # At this point, we default the errors to replace instead of strict
         # because nobody handles those errors anyways and at this point
         # we're so fundamentally fucked that nothing can repair it.
         if errors is None:
@@ -328,19 +374,143 @@ def get_streerror(e, default=None):
     return msg
 
 
-def open_stream(filename, mode='r', encoding=None, errors='strict'):
-    if filename != '-':
-        if encoding is not None:
-            return io.open(filename, mode, encoding=encoding,
-                           errors=errors), True
-        return open(filename, mode), True
-    if 'w' in mode:
+def open_stream(filename, mode='r', encoding=None, errors='strict',
+                atomic=False):
+    # Standard streams first.  These are simple because they don't need
+    # special handling for the atomic flag.  It's entirely ignored.
+    if filename == '-':
+        if 'w' in mode:
+            if 'b' in mode:
+                return get_binary_stdout(), False
+            return get_text_stdout(encoding=encoding, errors=errors), False
         if 'b' in mode:
-            return get_binary_stdout(), False
-        return get_text_stdout(encoding=encoding, errors=errors), False
-    if 'b' in mode:
-        return get_binary_stdin(), False
-    return get_text_stdin(encoding=encoding, errors=errors), False
+            return get_binary_stdin(), False
+        return get_text_stdin(encoding=encoding, errors=errors), False
+
+    # Non-atomic writes directly go out through the regular open functions.
+    if not atomic:
+        if encoding is None:
+            return open(filename, mode), True
+        return io.open(filename, mode, encoding=encoding, errors=errors), True
+
+    # Atomic writes are more complicated.  They work by opening a file
+    # as a proxy in the same folder and then using the fdopen
+    # functionality to wrap it in a Python file.  Then we wrap it in an
+    # atomic file that moves the file over on close.
+    import tempfile
+    fd, tmp_filename = tempfile.mkstemp(dir=os.path.dirname(filename),
+                                        prefix='.__atomic-write')
+
+    if encoding is not None:
+        f = io.open(fd, mode, encoding=encoding, errors=errors)
+    else:
+        f = os.fdopen(fd, mode)
+
+    return _AtomicFile(f, tmp_filename, filename), True
+
+
+# Used in a destructor call, needs extra protection from interpreter cleanup.
+_rename = os.rename
+
+
+class _AtomicFile(object):
+
+    def __init__(self, f, tmp_filename, real_filename):
+        self._f = f
+        self._tmp_filename = tmp_filename
+        self._real_filename = real_filename
+        self.closed = False
+
+    @property
+    def name(self):
+        return self._real_filename
+
+    def close(self, delete=False):
+        if self.closed:
+            return
+        self._f.close()
+        _rename(self._tmp_filename, self._real_filename)
+        self.closed = True
+
+    def __getattr__(self, name):
+        return getattr(self._f, name)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, tb):
+        self.close(delete=exc_type is not None)
+
+    def __repr__(self):
+        return repr(self._f)
+
+
+auto_wrap_for_ansi = None
+colorama = None
+
+
+# If we're on Windows, we provide transparent integration through
+# colorama.  This will make ANSI colors through the echo function
+# work automatically.
+if sys.platform.startswith('win'):
+    try:
+        import colorama
+    except ImportError:
+        pass
+    else:
+        _ansi_stream_wrappers = WeakKeyDictionary()
+
+        def auto_wrap_for_ansi(stream):
+            try:
+                cached = _ansi_stream_wrappers.get(stream)
+            except Exception:
+                cached = None
+            if cached is not None:
+                return cached
+            strip = not isatty(stream)
+            rv = colorama.AnsiToWin32(stream, strip=strip).stream
+            try:
+                _ansi_stream_wrappers[stream] = rv
+            except Exception:
+                pass
+            return rv
+
+
+def strip_ansi(value):
+    return _ansi_re.sub('', value)
+
+
+def term_len(x):
+    return len(strip_ansi(x))
+
+
+def isatty(stream):
+    try:
+        return stream.isatty()
+    except Exception:
+        return False
+
+
+_default_text_cache = WeakKeyDictionary()
+
+
+def _default_text_stdout():
+    """Like :func:`get_text_stdout` but uses a cache if available to speed
+    it up.  This will not create streams over and over again.
+    """
+    stream = sys.stdout
+    try:
+        rv = _default_text_cache.get(stream)
+    except Exception:
+        rv = None
+    if rv is not None:
+        return rv
+    rv = get_text_stdout()
+    try:
+        _default_text_cache[stream] = rv
+    except Exception:
+        pass
+    return rv
 
 
 binary_streams = {
