@@ -1,10 +1,9 @@
 import os
 import sys
 import stat
-import uuid
 
 from ._compat import open_stream, text_type, filename_to_ui, get_streerror
-from .exceptions import UsageError
+from .exceptions import BadParameter
 from .utils import safecall, LazyFile
 
 
@@ -17,7 +16,7 @@ class ParamType(object):
     *   it needs to convert from a string
     *   it needs to convert its result type through unchanged
         (eg: needs to be idempotent)
-    *   it needs to be able to deal with param and context being none.
+    *   it needs to be able to deal with param and context being `None`.
         This can be the case when the object is used with prompt
         inputs.
     """
@@ -27,10 +26,10 @@ class ParamType(object):
 
     #: if a list of this type is expected and the value is pulled from a
     #: string environment variable, this is what splits it up.  `None`
-    #: means any whitespace.  For all parameter the general rule is that
+    #: means any whitespace.  For all parameters the general rule is that
     #: whitespace splits them up.  The exception are paths and files which
-    #: are split by ``os.path.pathsep`` by default (":" on unix and ";' on
-    #: windows).
+    #: are split by ``os.path.pathsep`` by default (":" on Unix and ";" on
+    #: Windows).
     envvar_list_splitter = None
 
     def __call__(self, value, param=None, ctx=None):
@@ -40,6 +39,13 @@ class ParamType(object):
     def get_metavar(self, param):
         """Returns the metavar default for this param if it provides one."""
 
+    def get_missing_message(self, param):
+        """Optionally might return extra information about a missing
+        parameter.
+
+        .. versionadded:: 2.0
+        """
+
     def convert(self, param, ctx, value):
         """Converts the value.  This is not invoked for values that are
         `None` (the missing value).
@@ -50,20 +56,15 @@ class ParamType(object):
         """Given a value from an environment variable this splits it up
         into small chunks depending on the defined envvar list splitter.
 
-        If the splitter is set to `None` which means that whitespace splits,
-        then leading and trailing whitespace is ignored.  Otherwise leading
+        If the splitter is set to `None`, which means that whitespace splits,
+        then leading and trailing whitespace is ignored.  Otherwise, leading
         and trailing splitters usually lead to empty items being included.
         """
         return (rv or '').split(self.envvar_list_splitter)
 
     def fail(self, message, param=None, ctx=None):
         """Helper method to fail with an invalid value message."""
-        if param is None:
-            message = 'Invalid value: %s' % message
-        else:
-            names = ' / '.join(param.opts) or param.name
-            message = 'Invalid value for "%s": %s' % (names, message)
-        raise UsageError(message, ctx=ctx)
+        raise BadParameter(message, ctx=ctx, param=param)
 
 
 class FuncParamType(ParamType):
@@ -118,9 +119,22 @@ class Choice(ParamType):
     def get_metavar(self, param):
         return '[%s]' % '|'.join(self.choices)
 
+    def get_missing_message(self, param):
+        return 'Choose from %s.' % ', '.join(self.choices)
+
     def convert(self, value, param, ctx):
+        # Exact match
         if value in self.choices:
             return value
+
+        # Match through normalization
+        if ctx is not None and \
+           ctx.token_normalize_func is not None:
+            value = ctx.token_normalize_func(value)
+            for choice in self.choices:
+                if ctx.token_normalize_func(choice) == value:
+                    return choice
+
         self.fail('invalid choice: %s. (choose from %s)' %
                   (value, ', '.join(self.choices)), param, ctx)
 
@@ -215,6 +229,7 @@ class UUIDParameterType(ParamType):
     name = 'uuid'
 
     def convert(self, value, param, ctx):
+        import uuid
         try:
             return uuid.UUID(value)
         except ValueError:
@@ -232,7 +247,7 @@ class File(ParamType):
     Files can be opened for reading or writing.  The special value ``-``
     indicates stdin or stdout depending on the mode.
 
-    By default the file is opened for reading text data but it can also be
+    By default, the file is opened for reading text data, but it can also be
     opened in binary mode or for writing.  The encoding parameter can be used
     to force a specific encoding.
 
@@ -240,16 +255,23 @@ class File(ParamType):
     upon first IO.  The default is to be non lazy for standard input and
     output streams as well as files opened for reading, lazy otherwise.
 
+    Starting with click 2.0, files can also be opened atomically in which
+    case all writes go into a separate file in the same folder and upon
+    completion the file will be moved over to the original location.  This
+    is useful if a file regularly read by other users is modified.
+
     See :ref:`file-args` for more information.
     """
     name = 'filename'
     envvar_list_splitter = os.path.pathsep
 
-    def __init__(self, mode='r', encoding=None, errors='strict', lazy=None):
+    def __init__(self, mode='r', encoding=None, errors='strict', lazy=None,
+                 atomic=False):
         self.mode = mode
         self.encoding = encoding
         self.errors = errors
         self.lazy = lazy
+        self.atomic = atomic
 
     def resolve_lazy_flag(self, value):
         if self.lazy is not None:
@@ -268,16 +290,18 @@ class File(ParamType):
             lazy = self.resolve_lazy_flag(value)
 
             if lazy:
-                f = LazyFile(value, self.mode, self.encoding, self.errors)
+                f = LazyFile(value, self.mode, self.encoding, self.errors,
+                             atomic=self.atomic)
                 if ctx is not None:
                     ctx.call_on_close(f.close_intelligently)
                 return f
 
             f, should_close = open_stream(value, self.mode,
-                                          self.encoding, self.errors)
-            # If a context is provided we automatically close the file
+                                          self.encoding, self.errors,
+                                          atomic=self.atomic)
+            # If a context is provided, we automatically close the file
             # at the end of the context execution (or flush out).  If a
-            # context does not exist it's the caller's responsibility to
+            # context does not exist, it's the caller's responsibility to
             # properly close the file.  This for instance happens when the
             # type is used with prompts.
             if ctx is not None:
@@ -296,7 +320,7 @@ class File(ParamType):
 class Path(ParamType):
     """The path type is similar to the :class:`File` type but it performs
     different checks.  First of all, instead of returning a open file
-    handle it returns just the filename.  Secondly it can perform various
+    handle it returns just the filename.  Secondly, it can perform various
     basic checks about what the file or directory should be.
 
     :param exists: if set to true, the file or directory needs to exist for
@@ -395,6 +419,15 @@ def convert_type(ty, default=None):
         return FLOAT
     if guessed_type:
         return STRING
+
+    # Catch a common mistake
+    if __debug__:
+        try:
+            if issubclass(ty, ParamType):
+                raise AssertionError('Attempted to use an uninstantiated '
+                                     'parameter type (%s).' % ty)
+        except TypeError:
+            pass
     return FuncParamType(ty)
 
 
